// Iv2DViewer.h++

#ifndef _Iv2DViewer_H_
#define _Iv2DViewer_H_

#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/Xt/viewers/SoXtPlaneViewer.h>
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTransformSeparator.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/draggers/SoTranslate2Dragger.h>
#include <Inventor/draggers/SoScale2Dragger.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/Xt/SoXt.h>
#include <X11/Xm/Xm.h>
#include <X11/Vendor.h>
#include <X11/IntrinsicP.h>
#include <Inventor/SoPickedPoint.h>
#include <Inventor/events/SoMouseButtonEvent.h>
#include <Inventor/SoOffscreenRenderer.h>

typedef struct
{
    SbVec3f value;
    SoNode* object;
} Iv2DViewerCBData;

class Iv2DViewer
{
protected:
    void initialize( Widget = NULL );
    static void selectCB( void*, SoPath* );
    static void deselectCB( void*, SoPath* );
    static void keypressCB( void*, SoEventCallback* );
    static void mouseCB( void*, SoEventCallback* );
    SoPath* isObjectSelected( SoSeparator* );

    // ROOT
    //   |
    // 2DCAM...SELECT..........KEY-EVENT
    //           |
    //          SEP
    //           |
    //          SEP............SEP...........
    //           |              |          etc
    //     Xform...Obj1   Xform...Obj2

    SoXtPlaneViewer *renderArea;
    SoSeparator* root;
    SoOrthographicCamera* camera;
    SoSelection* selectRoot;
    SoSeparator* sceneRoot;
    SoEventCallback* keyEventNode;
    int (*userKeyCB)( void*, SoEventCallback* );
    void* userKeyData;
    int (*userMouseCB)( void*, SoEventCallback* );
    void* userMouseData;
    int altkey;
    int objectCount;
    int selectCount;
    SoSeparator* objRootSelected;
    int inited;
    // SoSeparator* transDraggerRoot;
    SoTransformSeparator* transDraggerRoot;
    SoTransform* transDraggerTransform;
    SoTranslate2Dragger* transDragger;
    static SoDraggerCB transDraggerValueCB;
    int (*userTransValueCB)( void*, Iv2DViewerCBData* );
    void* userTransValueData;

    SoTransformSeparator* scaleDraggerRoot;
    SoTransform* scaleDraggerTransform;
    SoScale2Dragger* scaleDragger;
    static SoDraggerCB scaleDraggerValueCB;
    int (*userScaleValueCB)( void*, Iv2DViewerCBData* );
    void* userScaleValueData;

public:

    Iv2DViewer( Widget = 0, int build = 1 );
    ~Iv2DViewer();
    int addObject( SoNode* );
    void removeObject( SoSeparator* );
    SoTransform* getObjectTransform( int );
    inline void setKeyCB( int (*)(void*, SoEventCallback*), void* = 0 );
    inline void setMouseCB( int (*)(void*, SoEventCallback*), void* = 0 );
    inline void setTransValueCB( int (*)(void*,Iv2DViewerCBData*), void* = 0 );
    inline void setScaleValueCB( int (*)(void*,Iv2DViewerCBData*), void* = 0 );
    inline void resize( int, int, int, int );
    inline SoSelection* getSelectRoot() { return selectRoot; }
    inline SoSeparator* getSceneRoot() { return sceneRoot; }
    inline SoOrthographicCamera* getCamera() { return camera; }
    inline int getObjectCount() { return objectCount; }
    inline SoSeparator* getSelectedObject() { return objRootSelected; }
    int printPS( FILE* fp, int whitebg = 0 );
};

inline 
void
Iv2DViewer::setKeyCB( int (*fn)(void*, SoEventCallback* ),
		      void* data )
{
    userKeyCB = fn;
    userKeyData = data;
}

inline 
void
Iv2DViewer::setMouseCB( int (*fn)(void*, SoEventCallback* ),
			void* data )
{
    userMouseCB = fn;
    userMouseData = data;
}

inline
void
Iv2DViewer::setTransValueCB( int (*fn)(void*,Iv2DViewerCBData*), 
			     void* data )
{
    userTransValueCB = fn;
    userTransValueData = data;
}

inline
void
Iv2DViewer::setScaleValueCB( int (*fn)(void*,Iv2DViewerCBData*), 
			     void* data )
{
    userScaleValueCB = fn;
    userScaleValueData = data;
}

inline
void
Iv2DViewer::resize( int x0, int y0, int w, int h )
{
    XtConfigureWidget( renderArea->getWidget(), x0, y0, w, h, 0 );
}

#endif /* _Iv2DViewer_H_ */
