// IvCameraOverlay.c++

#include "IvCameraOverlay.h++"

const int IvCameraOverlay::maxOverlayLines = 1000;
const int nColors = 3;

IvCameraOverlay::IvCameraOverlay()
  : lines( new IvCameraOverlayLine*[ maxOverlayLines ] ),
    nLines( 0 ),
    root( new SoSeparator ),
    camera( new SoOrthographicCamera ),
    lightModel( new SoLightModel ),
    colorIndex( new SoColorIndex ),
    linesRoot( new SoSeparator ),
    // lineColor( .8, .2, .3 )
    lineColors( new SbColor[nColors] ),
    textFieldCount( 0 ),
    textRoot( new SoSeparator )
{
    for( int i = 0; i < maxOverlayLines; i++ )
       lines[i] = 0;
    root -> ref();
    root -> addChild( camera );
    root -> addChild( lightModel );
    // root -> addChild( colorIndex );
    root -> addChild( linesRoot );
    camera -> height = .75;
    camera -> viewportMapping = SoCamera::CROP_VIEWPORT_FILL_FRAME;
    camera -> position.setValue( 0., 0., -.5 );
    camera -> orientation.setValue( 1., 0., 0., 0. );
    camera -> aspectRatio = 4. / 3.;
    camera -> nearDistance = 0.;
    camera -> farDistance = 1.;
    lightModel -> model.setValue( SoLightModel::BASE_COLOR );
    colorIndex -> index = 1;
    lineColors[0].setValue( .9, .1, .1 );
    // lineColors[1].setValue( .8, .8, .8 );
    lineColors[1].setValue( 1, 1, 1 );
    lineColors[2].setValue( .1, .9, .1 );
    // lineColors[3].setValue( .8, .2, .2 );
    // lineColors[4].setValue( .2, .8, .2 );
    // lineColors[5].setValue( .2, .2, .8 );
    root -> addChild( textRoot );
}

IvCameraOverlay::~IvCameraOverlay()
{
    root -> unref();
}

void
IvCameraOverlay::attachTo( SoXtRenderArea* ra )
{
    ra -> setOverlaySceneGraph( root );
    // ra -> setOverlayColorMap( 1, 1, &lineColor );
    ra -> setOverlayColorMap( 1, nColors, lineColors );
}

int
IvCameraOverlay::newLine( float x0, float y0, float x1, float y1, int c )
{
    if( nLines >= maxOverlayLines )
       return -1;
    lines[ nLines ] = new IvCameraOverlayLine( x0, y0, x1, y1, c );
    linesRoot -> addChild( lines[nLines]->getNode() );
    return( nLines++ );
}

void
IvCameraOverlay::redrawLine( int id, float x0, float y0, float x1, float y1,
			     int c )
{
    if( id < 0 || id >= nLines )
       return;
    lines[ id ] -> redraw( x0, y0, x1, y1, c );
}

#include <stream.h>

void
IvCameraOverlay::eraseLines( int startid )
{
    if( startid < 0 )
       return;
    for( int i = startid; i < nLines; i++ )
       {
	   linesRoot -> removeChild( startid );
	   delete lines[i];
       }
    nLines = startid;
}

void
IvCameraOverlay::setSize( int h, int w )
{
    camera -> height = float(h) / float(w);
    camera -> aspectRatio = float(w) / float(h);
}

int
IvCameraOverlay::newTextField()
{
    if( textFieldCount >= MAXTEXTFIELDS )
       return -1;
    IvCameraTextField* tf = new IvCameraTextField;
    textFields[ textFieldCount ] = tf;
    tf -> setLocation
       ( -.48, 
	 camera->height.getValue()/2 - .02 - textFieldCount*.035 );
    textRoot -> addChild( tf->getNode() );
    return( textFieldCount++);
}

void
IvCameraOverlay::setText( int which, char* s )
{
    if( which < 0 || which >= textFieldCount )
       return;
    textFields[which] -> setText( s );
}
