// qmSingularValueDecomposition.h++

#ifndef _qmSingularValueDecomposition_H_
#define _qmSingularValueDecomposition_H_

#include <stream.h>
#include <math.h>
#include "QuickMath/qmCompileOptions.h++"
#include "QuickMath/qmMatrixRef.h++"
#include "QuickMath/qmMatrix.h++"
#include "QuickMath/qmVectorRef.h++"
#include "QuickMath/qmVector.h++"

class qmSingularValueDecomposition
{
    qmMatrix Umatrix;
    qmVector singularValues;
    qmMatrix Vmatrix;

private:

    static real safePythag( const real, const real );
    static real svdMakeBidiagonal( qmMatrixRef& A, real rowvec[], real w[] );
    static qmMatrixRef& svdMakeRHXforms( qmMatrixRef& A,
					 real rowvec[],
					 qmMatrixRef& V );
    static qmMatrixRef& svdMakeLHXforms( qmMatrixRef& U, real w[] );
    static void svdMakeDiagonal( qmMatrixRef& U,
				 real w[],
				 qmMatrixRef& V,
				 real rowvec[],
				 real aNorm );

public:
    
    qmSingularValueDecomposition() {}
    qmSingularValueDecomposition( qmMatrixRef& m ) { set( m ); }
    qmSingularValueDecomposition& set( qmMatrixRef& );
    qmMatrixRef& getU() { return Umatrix; }
    qmMatrixRef& getV() { return Vmatrix; }
    qmVectorRef& getSingularValues() { return singularValues; }
    static real* svd( qmMatrixRef& A,
		      qmMatrixRef& U,
		      real w[],
		      qmMatrixRef& V );
    ostream& print( ostream& );
};

typedef qmSingularValueDecomposition qmSVD;

extern ostream& operator<<( ostream&, qmSingularValueDecomposition& );

#endif /* _qmSingularValueDecomposition_H_ */
