// qmVector.c++

#include "QuickMath/qmVector.h++"
#include "QuickMath/qmVectorRef.h++"
#include "QuickMath/qmCompileOptions.h++"

qmVector::qmVector( int n )
  : qmVectorRef( ), allocArray( 0 )
{
  set( n );
}

qmVector::qmVector( int n, real *data )
  : qmVectorRef( ), allocArray( 0 )
{
  set( n, data );
}

qmVector::qmVector( const qmVector& v )
  : qmVectorRef( ), allocArray( 0 )
{
  set( v );
}

qmVector::qmVector( const qmVectorRef& v )
  : qmVectorRef( ), allocArray( 0 )
{
  set( v );
}

qmVector&
qmVector::set( int n )
{
#ifdef QmDoBoundaryChecking
  QmAssert( n > 0 );
#endif
  if( !( allocArray && elementCount == n ) )
    {
      if( allocArray )
	delete [] allocArray;
      elementCount = n;
      allocArray = new real[ elementCount ];
    }
  array = allocArray;
  return *this;
}

qmVector&
qmVector::set( int n, real *data )
{
#ifdef QmDoBoundaryChecking
  QmAssert( n > 0 );
  QmAssert( data != (real *) NULL );
#endif
  set( n );
  setData( data );
  return *this;
}

qmVector&
qmVector::set( const qmVectorRef& qv )
{
#ifdef QmDoBoundaryChecking
  QmAssert( qv.elementCount > 0 );
  QmAssert( qv.array != (real *) NULL );
#endif
  if( !( allocArray && elementCount == qv.getElementCount() ) )
    {
      if( allocArray )
	delete [] allocArray;
      allocArray = new real[ qv.getElementCount() ];
      elementCount = qv.getElementCount();
      array = allocArray;
    }
  copy( qv );
  return *this;
}
