// $CopyrightHeader$

// Copyright (C) 1996 by Ali Azarbayejani and the Massachusetts
// Institute of Technology
//
// COMMERCIAL DISCLAIMER:
//
// This software is intended for research purposes only and contains
// unlicensed software, in original and modified forms, from various
// sources including the Free Software Foundation.  These programs
// shall not be used, rewritten, or adapted as the basis of a
// commercial software or hardware product without first obtaining
// appropriate licenses from the owners of the original software
// copyrights.
//
// In the case of copyrighted software fragments obtained from the
// Free Software Foundation: you can redistribute it and/or modify it
// under the terms of the GNU Library General Public License as
// published by the Free Software Foundation.  This software is
// distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General
// Public License for more details.  You should have received a copy
// of the GNU Library General Public License and the GNU General
// Public License along with this software; if not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.


// MIT COPYRIGHT:
// 
// Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
// rights reserved.
// 
// Developed  by Ali Azarbayejani at the Media Laboratory, MIT,
// Cambridge, Massachusetts.
// 
// Permission to use, copy,  or modify this software and  its documentation
// for  educational  and  research purposes only and without fee  is hereby
// granted, provided  that this copyright notice and the original authors's
// names appear  on all copies and supporting documentation.  If individual
// files are  separated from  this  distribution directory  structure, this
// copyright notice must be included.  For any other uses of this software,
// in original or  modified form, including but not limited to distribution
// in whole or in  part, specific  prior permission  must be  obtained from
// MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
// basis  of  a commercial  software  or  hardware product  without first
// obtaining appropriate licenses  from MIT.  MIT. makes no representations
// about the suitability of this  software for any purpose.  It is provided
// "as is" without express or implied warranty.

// ffinder.c++
#include "Parse/Parse.h++"
#include "vlImage/vlImage.h++"
#include "glDisplay/glDisplay.h++"
#include "Useless/HowSlow.h++"
#include "BlobMan.h++"
#include "FfinderServer.h++"
#include "Person.h++"
#include "FfFile.h++"
#include <stdio.h>
int main( int argc, char *argv[] )
{
    int verbose = 0;
    int myinput = 1;
    int zoom = 2;
    int width;
    int height;
    int aord = vlImage::ANALOG;
    int nbuffers = 20;
    int serveMode = 0;
    int processMode = 1;
    int undumpMode = 0;
    int displayMode = 1;
    int displayPointsMode = 1;
    int displayBlobsMode = 0;
    int displayQuickPoints = 1;
    int howslowMode = 0;
    int grabMode = 0;
    char* colorFile = 0;
    char* libPath = ".";
    ParseRuleFlag( 'v', verbose, 1, "Verbose mode" );
    ParseRuleFlag( 'v', BlobMan::verbose, 1, "Verbose mode" );
    ParseRuleTag( "vb", Blob::verbose, 1, "Verbose mode" );
    ParseRuleTag( "serve", serveMode, 1, "serve" );
    ParseRuleTag( "noproc", processMode, 0, "do not process" );
    ParseRuleTag( "nodpy", displayMode, 0, "do not display" );
    ParseRuleTag( "blobs", displayBlobsMode, 1, "display blobs" );
    ParseRuleTag( "howslow", howslowMode, 1, "howslow mode" );
    ParseRuleTag( "grab", grabMode, 1, "grab mode" );
    ParseRuleTaggedArg( "color", colorFile, "color stats file", "filename" );
    ParseRuleTag( "c", Blob::mode, (int)Blob::WOOHOO, "cell mode, woohoo" );
    ParseRuleTaggedArgInt( "c", BlobMan::cellWidth, "cell width", "w" );
    ParseRuleTaggedArgInt( "c", BlobMan::cellHeight, "cell height", "h" );
    ParseRuleTaggedArgFloat( "c", Cell::classifyPercentThresh, "thresh", 
			     "percent" );
    ParseRuleTaggedArgInt( "c", BlobMan::cullMinBlob, "medium min blob", "N" );
    int initMode = 0;
    ParseRuleTag( "init", initMode, 1, "init only mode" );
    char* ffFilename = "stive.both.ff";
    ParseRuleTaggedArg( "config", ffFilename, "filename of config data",
			"filename" );
    ParseRuleTaggedArgFloat( "crash", BlobMan::crashRatio, 
			     "minimum crash pixels for collide", "ratio" );
    ParseRuleTaggedArg( "path", libPath, "library path", "path" );
    ParseRuleTaggedArgInt( "et", BlobMan::existenceThresh, 
			   "existence threshold", "number" );
    if( Parse::ParseCmdLine( argc, argv ) )
       Parse::Usage( argv[0] ), exit(1);
    FfFile ffFile;
    ffFile.setPath( libPath );
    if( ffFile.scan( ffFilename ) )
       exit( 1 );
    if( grabMode )
       {
	   vliYUVImageRef iref;
	   vlImage::init( myinput, zoom, vlImage::YUV, aord, nbuffers );
	   sginap( 50 );
	   vlImage::grab( iref );
	   iref.writeRawRGB( cout );
	   vlImage::free();
	   cerr << "wrote image, w= " << iref.getWidth() 
		<< ", h= " << iref.getHeight() << endl;
	   exit( 0 );
       }
    FfinderServer ffinderServer;
    if( serveMode )
       {
	   ffinderServer.init();
	   ffinderServer.start();
       }
    vliYUVImageRef iref;
    vlImage::init( myinput, zoom, vlImage::YUV, aord, nbuffers );
    vlImage::grab( iref );
    width = iref.getWidth();
    height = iref.getHeight();
    vliRGBImage rgbImage( width, height );
    glDisplay gldpy;
    if( displayMode ) gldpy.init( height, width );
    vlImage::free();
    BlobMan blobMan;
    blobMan.init( width, height );
    if( initMode )
       blobMan.initOnly();
    blobMan.setFleshStats( qmVectorRef( 3, ffFile.colorMean ),
			   qmMatrixRef( 3, 3, ffFile.colorInvCov ) );
    HowSlow howSlow;
    Person person;
    int processStatus = 0;
    static struct timeval tv;
    for( int frameCount = 0;; frameCount++ )
       {
	   if( howslowMode )
	      howSlow.sample();
	   if( !undumpMode )
	      vlImage::grab( iref );
	   if( FfinderServer::forceInit )
	      {
		  FfinderServer::forceInit = 0;
		  blobMan.setMode( blobMan.INIT_MODE );
	      }
	   if( processMode )
	      processStatus = blobMan.process( iref );
	   gettimeofday( &tv );
	   if( blobMan.INIT_MODE == blobMan.getMode() || processStatus )
	      person.init( frameCount, tv.tv_sec, tv.tv_usec,
			   blobMan.getPersonExists() );
	   else
	      person.set( frameCount, tv.tv_sec, tv.tv_usec, 
			  blobMan.getPersonExists(),
			  blobMan.getBlobMean_Pics( blobMan.HEAD ),
			  blobMan.getBlobMean_Pics( blobMan.LHAND ),
			  blobMan.getBlobMean_Pics( blobMan.RHAND ) );
	   if( displayMode ) 
	      {
		  iref._toabgr( rgbImage.getPixelData() );
		  if( displayBlobsMode )
		     blobMan.drawBlobs( rgbImage );
		  else if( displayPointsMode )
		     blobMan.drawPoints( rgbImage );
		  if( displayQuickPoints && 
		      blobMan.INIT_MODE != blobMan.getMode() )
		     {
			 static vliRGBPixel white( 255, 255, 255 );
			 float qp[12];
			 memcpy
			    ( qp, blobMan.getQuickPoints(), 12*sizeof(float) );
			 for( int i = 0; i < 6; i++ )
			    rgbImage.drawCross( qp[2*i], qp[2*i+1], white );
		     }
		  gldpy = (unsigned long *) rgbImage.getPixelPtr(0);
	      }
	   if( serveMode ) ffinderServer.copy( person.get() );
vlImage::free(); } }
