// PersonPlus.c++

#include "PersonPlus.h++"
#include "Useless/iotools.h++"

int PersonPlus::maxRingLength = 500;
real PersonPlus::moveThresh = .05;

PersonPlus::PersonPlus() 
  : person( 0 ),
    ring( 0 ),
    ringLength( 0 ),
    ringAllocLength( 0 ),
    yInit( 0 ),
    ySync( 0 ),
    personState( NoPerson ),
    correspond( 0 )
{
    initRing();
    person = ring;
    // ringLength = 1;
    ringLength = 0;
    person -> frameNumber = -1;
    person -> timestampSec = 0;
    person -> timestampUsec = 0;
    person -> head.valid = 0;
    person -> leftHand.valid = 0;
    person -> rightHand.valid = 0;
}

// static 
// inline
// istream&
// scanVector( istream& is, double* v, int n )
// {
//     while( n-- )
//        is >> *v++;
//     return is;
// }

static
inline
istream&
scanImagePoint( istream& is, sffImagePoint& ip )
{
    is >> ip.valid;
    iotools_scanVector( is, ip.t, 2 );
    iotools_scanVector( is, ip.c, 4 );
    is >> ip.angle;
    iotools_scanVector( is, ip.s, 2 );
    return is;
}

static
inline
istream&
scanWorldBlob( istream& is, sffWorldBlob& wb )
{
    iotools_scanVector( is, wb.t, 3 );
    iotools_scanVector( is, wb.q, 4 );
    iotools_scanVector( is, wb.s, 3 );
    return is;
}

static
inline
istream&
scanWorldPart( istream& is, sffWorldPart& wp )
{
    is >> wp.valid;
    scanWorldBlob( is, wp.camera_ref );
    scanWorldBlob( is, wp.world_ref );
    scanImagePoint( is, wp.left );
    scanImagePoint( is, wp.right );
    return is;
}

static
istream&
scanPersonPlus( istream& is, sffPersonPlus& pp )
{
    is >> pp.frameNumber;
    is >> pp.timestampSec;
    is >> pp.timestampUsec;
    is >> pp.personExists;
    scanWorldPart( is, pp.head );
    scanWorldPart( is, pp.leftHand );
    scanWorldPart( is, pp.rightHand );
    return is;
}

// static 
// inline
// ostream&
// printVector( ostream& os, double* v, int n, char* pre = "" )
// {
//     os << pre;
//     while( n-- )
//        os << *v++ << " ";
//     return os;
// }

static
inline
ostream&
printImagePoint( ostream& os, sffImagePoint& ip, char* pre = "" )
{
    os << pre << ip.valid << endl;
    iotools_printVector( os << "      ", ip.t, 2 ) << endl;
    iotools_printVector( os << "      ", ip.c, 4 ) << endl;
    os << "      " << pre << ip.angle << endl;
    iotools_printVector( os << "      ", ip.s, 2 ) << endl;
    return os;
}

static
inline
ostream&
printWorldBlob( ostream& os, sffWorldBlob& wb )
{
    iotools_printVector( os << "    ", wb.t, 3 ) << endl;
    iotools_printVector( os << "    ", wb.q, 4 ) << endl;
    iotools_printVector( os << "    ", wb.s, 3 ) << endl;
    return os;
}

static
inline
ostream&
printWorldPart( ostream& os, sffWorldPart& wp )
{
    os << "  " << wp.valid << endl;
    printWorldBlob( os, wp.camera_ref );
    printWorldBlob( os, wp.world_ref );
    printImagePoint( os, wp.left, "    " );
    printImagePoint( os, wp.right, "    " );
    return os;
}

static
ostream&
printPersonPlus( ostream& os, sffPersonPlus& pp )
{
    os << pp.frameNumber << " ";
    os << pp.timestampSec << " ";
    os << pp.timestampUsec << " ";
    os << pp.personExists << endl;
    printWorldPart( os, pp.head );
    printWorldPart( os, pp.leftHand );
    printWorldPart( os, pp.rightHand );
    os << endl;
    return os;
}

void
PersonPlus::initRing( int n )
{
    if( ring && n > ringAllocLength )
       {
	   delete ring;
	   ring = 0;
       }
    if( !ring )
       {
	   ring = new sffPersonPlus[ n ];
	   ringAllocLength = n;
       }
    ringLength = 0;
    person = ring;
    ySync = 0;
}

istream&
PersonPlus::readRing( istream& is )
{
    initRing(); // ! resets to maximum length
    person = ring;
    for( ringLength = 0; // ! not necessary since initRing() resets ringLength
	 ringLength < ringAllocLength && scanPersonPlus( is, *person++ ); 
	 ringLength++ );
    if( ringLength >= ringAllocLength && !is.eof() )
       alert( "readRing", "ringAllocLength reached" ) 
	  << "\tringAllocLength= " << ringAllocLength << endl;
    person = ring;
    ySync = 0;
    if( ringLength < 2 )
       error( "readRing", "ringLength < 2" );
//     dt_k = Time( (person+1)->timestampSec, (person+1)->timestampUsec ) -
//        Time( person->timestampSec, person->timestampUsec );
    dt_k = qmTime( (person+1)->timestampSec, (person+1)->timestampUsec ) -
       qmTime( person->timestampSec, person->timestampUsec );
    // ! bad news if ringLength < 2 or something
//     time_prev.Set();
    time_prev.set();
    return is;
}

int
PersonPlus::readRing( char* filename )
{
    ifstream filestream( filename );
    readRing( filestream );
    if( filestream.fail() || filestream.bad() )
       return 1;
    filestream.close();
    return 0;
}

ostream&
PersonPlus::writeRing( ostream& os )
{
    for( int i = 0; i < ringLength; i++ )
       printPersonPlus( os, ring[i] );
    return os;
}

int
PersonPlus::writeRing( char* filename )
{
    ofstream filestream( filename );
    writeRing( filestream );
    if( filestream.fail() || filestream.bad() )
       return 1;
    filestream.close();
    return 0;
}

inline
void
PersonPlus::nextInRing_force()
{
    // assumes ring exists and is valid
    // records do not have to be valid
    // pays no attention to time
    person++;
    if( person - ring >= ringLength )
       person = ring;
    ySync = 0;
}

inline
void
PersonPlus::prevInRing_force()
{
    // assumes ring exists and is valid
    // records do not have to be valid
    // pays no attention to time
    person--;
    if( person - ring < 0 )
       person = ring + ringLength-1;
    ySync = 0;
}

inline
void
PersonPlus::nextInRing_timed()
{
    // assumes ring exists and is valid
    // assumes time_prev, dt_k exist
    // assumes records are valid (to compute times
//     time_now.Set();
    time_now.set();
    dt_now = time_now - time_prev;
    if( dt_now < dt_k )
       return;
    person++;
    time_prev = time_now;
    if( person - ring >= ringLength )
       person = ring;
    if( person - ring == ringLength-1 )
       dt_k.set( 0, 0 );
    else
       dt_k = qmTime( (person+1)->timestampSec, (person+1)->timestampUsec ) -
	  qmTime( person->timestampSec, person->timestampUsec );
    ySync = 0;
}

int
PersonPlus::dump()
{
    if( !ring )
       error( "dump", "ring not initialized" );
    if( ringLength >= ringAllocLength-1 )
       return 1;
    person++;
    ringLength++;
    dt_k = qmTime( 0, 0 );
//     time_prev.Set();
    time_prev.set();
    // nextInRing_force();
    // if( person == ring )
    // return 1;
    return 0;
}

int
PersonPlus::undump_timed()
{
    if( !ring )
       error( "undump_timed", "ring not initialized" );
    nextInRing_timed();
    return 0;
}

int
PersonPlus::undump_force( int dir )
{
    if( !ring )
       error( "undump_force", "ring not initialized" );
    if( Forward == dir )
       nextInRing_force();
    else if( Backward == dir )
       prevInRing_force();
    else
       return 1;
    return 0;
}

static qmVector evalues( 2 );
static qmMatrix evectors( 2, 2 );  // evectors in rows (dumb)

inline
void
fillItUp( sffImagePoint& point, const ffImagePoint& fpoint )
{
    point.valid = fpoint.valid;
    point.t[0] = fpoint.u;
    point.t[1] = fpoint.v;
    point.c[0] = fpoint.suu;
    point.c[1] = fpoint.suv;
    point.c[2] = fpoint.suv;
    point.c[3] = fpoint.svv;
    // 2d stats
    qmEigenStructure::eigenSymmetric2( qmMatrixRef( 2, 2, point.c ),
				       evalues, evectors );
    real& theSine = evectors(0,1);
    real& theCosine = evectors(0,0);
    point.angle = atan2( theSine, theCosine );
    point.s[0] = 2. * sqrt( evalues(0) );
    point.s[1] = 2. * sqrt( evalues(1) );
}

inline 
void
fillItUp( sffPersonPlus& sperson,
	  const ffPerson& lperson, 
	  const ffPerson& rperson )
{
    fillItUp( sperson.head.left, lperson.head );
    fillItUp( sperson.head.right, rperson.head );
    sperson.head.valid = lperson.head.valid && rperson.head.valid;
    fillItUp( sperson.leftHand.left, lperson.leftHand );
    fillItUp( sperson.leftHand.right, rperson.leftHand );
    sperson.leftHand.valid = lperson.leftHand.valid && rperson.leftHand.valid;
    fillItUp( sperson.rightHand.left, lperson.rightHand );
    fillItUp( sperson.rightHand.right, rperson.rightHand );
    sperson.rightHand.valid = 
       lperson.rightHand.valid && rperson.rightHand.valid;
}

static
inline
real
computeMoveMag( real* last, const ffImagePoint& fpoint )
{
    real du = ( last[0] - fpoint.u );
    real dv = ( last[1] - fpoint.v );
    real mag = ( du*du + dv*dv );
    last[0] = fpoint.u;
    last[1] = fpoint.v;
    return mag;
}

void
PersonPlus::set2( const ffPerson& left, const ffPerson& right )
{
    moveMags[0] = computeMoveMag( lastImagePos[0][0], left.head );
    moveMags[1] = computeMoveMag( lastImagePos[0][1], left.leftHand );
    moveMags[2] = computeMoveMag( lastImagePos[0][2], left.rightHand );
    moveMags[3] = computeMoveMag( lastImagePos[1][0], left.head );
    moveMags[4] = computeMoveMag( lastImagePos[1][1], left.leftHand );
    moveMags[5] = computeMoveMag( lastImagePos[1][2], left.rightHand );
    moveMag = sqrt( qmVectorRef( 6, moveMags ).getMax() );
	   
    fillItUp( *person, left, right );
    person -> personExists = left.personExists && right.personExists;
    ySync = 0;
}

qmVectorRef* 
PersonPlus::getYArray()
{
    if( !yInit )
       {
	   for( int i = 0; i < 3; i++ )
	      {
		  yArray[i] = new qmVector( 10 );
		  yRefArray[i].set( *yArray[i] );
	      }
	   yInit = 1;
       }
    if( !ySync )
       {
	   for( int i = 0; i < 3; i++ )
	      {
		  sffWorldPart& wp = (HEAD==i) ? person->head
		     : (LEFTHAND==i) ? person->leftHand
		     : person->rightHand;
		  qmVectorRef& dst = *yArray[i];
		  dst( 0 ) = wp.left.t[ 0 ];
		  dst( 1 ) = wp.left.t[ 1 ];
		  dst( 2 ) = wp.left.c[ 0 ];
		  dst( 3 ) = wp.left.c[ 3 ];
		  dst( 4 ) = wp.left.c[ 1 ];
		  dst( 5 ) = wp.right.t[ 0 ];
		  dst( 6 ) = wp.right.t[ 1 ];
		  dst( 7 ) = wp.right.c[ 0 ];
		  dst( 8 ) = wp.right.c[ 3 ];
		  dst( 9 ) = wp.right.c[ 1 ];
	      }
	   ySync = 1;
       }
    return yRefArray;
}

int*
PersonPlus::getValid()
{
    valid[0] = person -> head.valid;
    valid[1] = person -> leftHand.valid;
    valid[2] = person -> rightHand.valid;
    return valid;
}

qmVectorRef*
PersonPlus::getTArray()
{
    for( int i = 0; i < 3; i++ )
       {
	   sffWorldPart& wp = (HEAD==i) ? person->head
	      : (LEFTHAND==i) ? person->leftHand
	      : person->rightHand;
	   tRefArray[i].set( 3, wp.camera_ref.t );
       }
    return tRefArray;
}

qmUnitQuaternionRef* 
PersonPlus::getQArray()
{
    for( int i = 0; i < 3; i++ )
       {
	   sffWorldPart& wp = (HEAD==i) ? person->head
	      : (LEFTHAND==i) ? person->leftHand
	      : person->rightHand;
	   qRefArray[i].set( wp.camera_ref.q );
       }
    return qRefArray;
}

qmVectorRef*
PersonPlus::getLArray()
{
    for( int i = 0; i < 3; i++ )
       {
	   sffWorldPart& wp = (HEAD==i) ? person->head
	      : (LEFTHAND==i) ? person->leftHand
	      : person->rightHand;
	   lRefArray[i].set( 3, wp.camera_ref.s );
       }
    return lRefArray;
}

qmVectorRef*
PersonPlus::getTWArray()
{
    for( int i = 0; i < 3; i++ )
       {
	   sffWorldPart& wp = (HEAD==i) ? person->head
	      : (LEFTHAND==i) ? person->leftHand
	      : person->rightHand;
	   tWRefArray[i].set( 3, wp.world_ref.t );
       }
    return tWRefArray;
}

qmUnitQuaternionRef* 
PersonPlus::getQWArray()
{
    for( int i = 0; i < 3; i++ )
       {
	   sffWorldPart& wp = (HEAD==i) ? person->head
	      : (LEFTHAND==i) ? person->leftHand
	      : person->rightHand;
	   qWRefArray[i].set( wp.world_ref.q );
       }
    return qWRefArray;
}

qmVectorRef*
PersonPlus::getLWArray()
{
    for( int i = 0; i < 3; i++ )
       {
	   sffWorldPart& wp = (HEAD==i) ? person->head
	      : (LEFTHAND==i) ? person->leftHand
	      : person->rightHand;
	   lWRefArray[i].set( 3, wp.world_ref.s );
       }
    return lWRefArray;
}

// qmVectorRef*
// PersonPlus::getCArray()
// {
//     for( int i = 0; i < 3; i++ )
//        {
// 	   sffWorldPart& wp = (HEAD==i) ? person->head
// 	      : (LEFTHAND==i) ? person->leftHand
// 	      : person->rightHand;
// 	   cRefArray[i].set( 9, wp.c );
//        }
//     return cRefArray;
// }

// inline
// void
// PersonPlus::syncFloat()
// {
//     for( int i = 0; i < 3; i++ )
//        {
// 	   sffWorldPart& wp = (HEAD==i) ? person->head
// 	      : (LEFTHAND==i) ? person->leftHand
// 	      : person->rightHand;
// 	   tFloatArray[i][0] = wp.t[0];
// 	   tFloatArray[i][1] = wp.t[1];
// 	   tFloatArray[i][2] = wp.t[2];
// 	   qFloatArray[i][0] = wp.q[0];
// 	   qFloatArray[i][1] = wp.q[1];
// 	   qFloatArray[i][2] = wp.q[2];
// 	   qFloatArray[i][3] = wp.q[3];
// 	   lFloatArray[i][0] = wp.s[0];
// 	   lFloatArray[i][1] = wp.s[1];
// 	   lFloatArray[i][2] = wp.s[2];
//        }
//     floatSync = 1;
// }
// 
// float**
// PersonPlus::getTFloatArray()
// {
//     if( !floatSync )
//        syncFloat();
//     return tFloatArray;
// }
// 
// float**
// PersonPlus::getQFloatArray()
// {
//     if( !floatSync )
//        syncFloat();
//     return qFloatArray;
// }
// 
// float**
// PersonPlus::getLFloatArray()
// {
//     if( !floatSync )
//        syncFloat();
//     return lFloatArray;
// }

void
PersonPlus::findHands()
{
    hands.frameNumber = person -> frameNumber;
    hands.timestampSec = person -> timestampSec;
    hands.timestampUsec = person -> timestampUsec;
    hands.personExists = person -> personExists;
    const real root5 = sqrt( 5 );

    getValid();
    getTWArray();
    getQWArray();
    getLWArray();

    hands.head.valid = valid[ HEAD ];
    hands.head.x = tWRefArray[ HEAD ].getValue( 0 );
    hands.head.y = tWRefArray[ HEAD ].getValue( 1 );
    hands.head.z = tWRefArray[ HEAD ].getValue( 2 );

    // find ends of hand blobs
    // first, find the max dim
    // compute the two extrema in blob coords
    // rotate and translate to camera coords
    // find out which one is furthest from head; use that one
    
    // int maxi;
    real maxv;
    qmVector ex0( 3 );
    qmVector ex1( 3 );
    qmVector tmp( 3 );
    qmMatrix R( 3, 3 );
    real dist0;
    real dist1;
    qmVector delta(3);
    qmVectorRef farthest;
    sffWorldPoint* wp;
    for( int i = 1; i < 3; i++ ) // LEFTHAND and RIGHTHAND
       {
	   maxv = lWRefArray[i].getMax( &handWhichAxis[i-1] );
	   ex0.setZero();
	   ex0( handWhichAxis[i-1] ) = root5 * maxv;
	   qWRefArray[i].getRotationMatrix( R );
	   tmp.setMultiply( R, ex0 );
	   ex0.setAdd( tWRefArray[i], tmp );
	   ex1.setSubtract( tWRefArray[i], tmp );
	   delta.setSubtract( tWRefArray[HEAD], ex0 );
	   dist0 = sqrt( delta.getInnerProduct( delta ) );
	   delta.setSubtract( tWRefArray[HEAD], ex1 );
	   dist1 = sqrt( delta.getInnerProduct( delta ) );
	   farthest = ( dist0 > dist1 ) ? ex0 : ex1;
	   handMinus[i-1] = ( dist0 > dist1 ) ? 0 : 1;
	   wp = ( LEFTHAND==i ) ? &hands.leftHand : &hands.rightHand;
	   wp -> valid = valid[i];
	   wp -> x = farthest(0);
	   wp -> y = farthest(1);
	   wp -> z = farthest(2);
       }
}

// const qmVectorRef&
// PersonPlus::getLeftHand()
// {
//     static qmVector v(3);
//     v(0) = hands.leftHand.x;
//     v(1) = hands.leftHand.y;
//     v(2) = hands.leftHand.z;
//     return v;
// }
// 
// const qmVectorRef&
// PersonPlus::getRightHand()
// { 
//     static qmVector v(3);
//     v(0) = hands.rightHand.x;
//     v(1) = hands.rightHand.y;
//     v(2) = hands.rightHand.z;
//     return v;
// }

void 
PersonPlus::setState()
{
    // simple version
    if( person->head.valid && 
	person->leftHand.valid &&
	person->rightHand.valid )
       personState = ValidPerson;
    else
       {
	   personState = NoPerson;
	   // alert( "setState", "NoPerson" );
       }
}

ostream&
PersonPlus::alert( char* fn, char* msg )
{
    return( cerr << "PersonPlus: ALERT: " << fn << "(): " << msg << endl );
}

void
PersonPlus::error( char* fn, char* msg )
{
    cerr << "PersonPlus: ERROR: " << fn << "(): " << msg << endl;
    // exit( 1 );
    assert(0);
}

static
inline
void
copyCorr( float* p, sffWorldPart& wp )
{
    p[0] = wp.left.t[0];
    p[1] = wp.left.t[1];
    p[2] = wp.right.t[0];
    p[3] = wp.right.t[1];
}

float*
PersonPlus::getRingCorr( int& ncorr )
{
    if( !correspond )
       correspond = new float[ 12 * maxRingLength ];
    float* record = correspond;
    ncorr = 0;
    for( int i = 0; i < ringLength; i++ )
       {
	   sffPersonPlus& pp = ring[i];
	   if( pp.head.valid )
	      copyCorr( record, pp.head ), record += 4, ncorr++;
	   if( pp.leftHand.valid )
	      copyCorr( record, pp.leftHand ), record += 4, ncorr++;
	   if( pp.rightHand.valid )
	      copyCorr( record, pp.rightHand ), record += 4, ncorr++;
       }
    return correspond;
}
