// PersonPlus.h++

#ifndef _PersonPlus_H_
#define _PersonPlus_H_

#include <rpc/rpc.h>
#include "sffinder_svc.h"
#include "ffinder_clnt.h"
#define QmDouble
#include "QuickMath/QuickMath.h++"
// #include "Time/Time.h++"
#include "QuickMath/qmTime.h++"

class PersonPlus
{
    inline void nextInRing_timed();
    inline void nextInRing_force();
    inline void prevInRing_force();

    sffPersonPlus* person;
    sffPersonPlus* ring;
    int ringLength;
    int ringAllocLength;
//     Time time_now;
//     Time time_prev;
//     Time dt_now;
//     Time dt_k;
    qmTime time_now;
    qmTime time_prev;
    qmTime dt_now;
    qmTime dt_k;
    sffPerson hands;
    int handWhichAxis[2];
    int handMinus[2];
    real quickPoints[ 3*6 ];

    int valid[3];
    qmVector* yArray[3];
    int yInit;
    qmVectorRef yRefArray[3];
    int ySync; // y is consistent with "person"
    qmVectorRef tRefArray[3];
    qmUnitQuaternionRef qRefArray[3];
    qmVectorRef lRefArray[3];
    qmVectorRef tWRefArray[3];
    qmUnitQuaternionRef qWRefArray[3];
    qmVectorRef lWRefArray[3];
    static ostream& alert( char* fn, char* msg );
    static void error( char* fn, char* msg );

    real lastImagePos[2][3][2];
    real moveMags[6];
    real moveMag;

public:
    static int maxRingLength;

    PersonPlus();

    enum { Forward, Backward };
    void initRing( int n = maxRingLength );
    istream& readRing( istream& );
    int readRing( char* filename );
    ostream& writeRing( ostream& );
    int writeRing( char* filename );
    int dump();
    int undump_force( int dir = Forward );
    int undump_timed();

    const sffPersonPlus& get() { return *person; }
    enum { HEAD=0, LEFTHAND=1, RIGHTHAND=2 };
    void set2( const ffPerson& left, const ffPerson& right );
    qmVectorRef* getYArray();
    int* getValid();
    qmVectorRef* getTArray();
    qmUnitQuaternionRef* getQArray();
    qmVectorRef* getLArray();
    qmVectorRef* getTWArray();
    qmUnitQuaternionRef* getQWArray();
    qmVectorRef* getLWArray();
    // qmVectorRef* getCArray();
    inline void setStamp( long n, long s, long us );
    void findHands();
    const sffPerson& getHands() const { return hands; }
//     const qmVectorRef& getLeftHand();
//     const qmVectorRef& getRightHand();
    int* getHandWhichAxis() { return handWhichAxis; }
    int* getHandMinus() { return handMinus; }

    enum { NoPerson, ValidPerson, InvalidPerson };
    int personState;
    void setState();
    inline int getState() { return personState; }

    inline int getFrameNumber() { return person->frameNumber; }
    inline int personExists() { return person->personExists; }
    inline int personFullyValid();

    static real moveThresh;
    inline int moved2D() { return( moveMag > moveThresh ); }

    float* correspond;
    float* getRingCorr( int& n );
    real* getQPArray() { return quickPoints; }
    int getRingLength() { return ringLength; }
};

inline
void
PersonPlus::setStamp( long n, long s, long us )
{
    person -> frameNumber = n;
    person -> timestampSec = s;
    person -> timestampUsec = us;
}

inline
int
PersonPlus::personFullyValid()
{
    getValid();
    if( valid[0] && valid[1] && valid[2] )
       return 1;
    return 0;
}

#endif /* _PersonPlus_H_ */
