// SffFile.c++

#include "SffFile.h++"
#include "Useless/iotools.h++"
#include <assert.h>

static char fullpath[512];
static 
inline
char*
makeFullPath( char* pname, char* fname )
{
    if( fname[0] == '/' )
       sprintf( fullpath, fname );
    else
       sprintf( fullpath, "%s/%s", pname, fname );
    return fullpath;
}

inline
istream&
SffFile::scan_hosts( istream& is )
{
    return( is >> ffinder_left >> ffinder_right );
}

inline
istream&
SffFile::scan_stereo( istream& is )
{
    iotools_scanVector( is, stereo_t, 3 );
    iotools_scanVector( is, stereo_q, 4 );
    is >> stereo_bl;
    is >> stereo_br;
    return is;
}

inline
istream&
SffFile::scan_view( istream& is )
{
    iotools_scanVector( is, view_t, 3 );
    iotools_scanVector( is, view_q, 4 );
    is >> view_heightAngle;
    return is;
}

inline
istream&
SffFile::scan_world( istream& is )
{
    iotools_scanVector( is, world_t, 3 );
    iotools_scanVector( is, world_r, 9 );
    is >> world_scale;
    return is;
}

istream& 
SffFile::scan( istream& is )
{
    char filename[512];
    ifstream file;
    scan_hosts( is );
    if( iotools::wordpeek( is, "FILE" ) )
       {
	   is >> filename >> filename;
	   makeFullPath( pathname, filename );
	   file.open( fullpath );
	   if( !file )
	      {
		  alert( "scan", "can't open file" ) << "\t" 
						     << fullpath << endl;
		  error( "scan", "open failed" );
	      }
	   scan_stereo( file );
	   file.close();
       }
    else
       scan_stereo( is );
    if( iotools::wordpeek( is, "FILE" ) )
       {
	   is >> filename >> filename;
	   makeFullPath( pathname, filename );
	   file.open( fullpath );
	   if( !file )
	      {
		  alert( "scan", "can't open file" ) << "\t" 
						     << fullpath << endl;
		  error( "scan", "open failed" );
	      }
	   scan_view( file );
	   file.close();
       }
    else
       scan_view( is );
    if( iotools::wordpeek( is, "FILE" ) )
       {
	   is >> filename >> filename;
	   makeFullPath( pathname, filename );
	   file.open( fullpath );
	   if( !file )
	      {
		  alert( "scan", "can't open file" ) << "\t" 
						     << fullpath << endl;
		  error( "scan", "open failed" );
	      }
	   scan_world( file );
	   file.close();
       }
    else
       scan_world( is );
    return is;
}

int
SffFile::scan( char* filename )
{
    makeFullPath( pathname, filename );
    ifstream file( fullpath );
    if( !file )
       {
	   alert( "scan", "can't open file" ) << "\t" 
					      << fullpath << endl;
	   error( "scan", "open failed" );
       }
    scan( file );
    file.close();
    return 0;
}

ostream& 
SffFile::print( ostream& os )
{
    os << ffinder_left << endl;
    os << ffinder_right << endl;
    os << endl;
    iotools_printVector( os, stereo_t, 3 ) << endl;
    iotools_printVector( os, stereo_q, 4 ) << endl;
    os << stereo_bl << endl;
    os << stereo_br << endl;
    os << endl;
    iotools_printVector( os, view_t, 3 ) << endl;
    iotools_printVector( os, view_q, 4 ) << endl;
    os << view_heightAngle << endl;
    os << endl;
    iotools_printVector( os, world_t, 3 ) << endl;
    iotools_printVector( os, world_r, 9 ) << endl;
    os << world_scale << endl;
    return os;
}

int
SffFile::print( char* filename )
{
    makeFullPath( pathname, filename );
    ofstream file( fullpath );
    print( file );
    file.close();
    return 0;
}

ostream&
SffFile::alert( char* fn, char* msg )
{
    return( cerr << "SffFile: ERROR: " << fn << "(): " << msg << endl );
}

void
SffFile::error( char* fn, char* msg )
{
    alert( fn, msg );
    // exit( 1 );
    assert(0);
}
