// SffinderClient.c++

#include "SffinderClient.h++"
#include <unistd.h>
#include <limits.h>
#include <stream.h>
#include "Useless/iotools.h++"

SffinderClient::SffinderClient( char* serverHostname )
  : client( 0 ), lastFrame( -1 ), lastGot( GOTNONE )
{
    person.rightHand.valid = 0;
    person.leftHand.valid = 0;
    person.head.valid = 0;
    if( serverHostname )
       if( connect( serverHostname ) )
	  cerr << "connection failed" << endl;
}

int
SffinderClient::connect( char* serverHostname )
{
    const int tryMax = 100;
    const int tryInterval = CLK_TCK;
    int tryCount = 0;

    cerr << "connecting to sffinder server on " << serverHostname << endl;
    while( tryCount++ < tryMax &&
	   !( client = clnt_create
	      ( serverHostname, SFFINDERPROG, SFFINDERVERS, "tcp") ) )
       {
	   clnt_pcreateerror( serverHostname );
	   sginap( 100 );
       }
    if( tryCount >= tryMax )
       return 1;
    cerr << "connection successful" << endl;
    return 0;
}

// int
// SffinderClient::getPersonFromServer()
// {
//     sffPerson* p = 0;
// 
//     if( !client )
//        return 1;
//     if( !( p = sffgetperson_1( 0, client ) ) )
//        {
// 	   clnt_perror( client, "sffgetperson_1" );
// 	   return 2;
//        }
//     person = *p;
//     return 0;
// }

int
SffinderClient::getPersonFromServer()
{
    sffPersonPlus* p = 0;
    if( !client )
       return 1;
    if( !( p = sffgetpersonplus_1( 0, client ) ) )
       {
	   clnt_perror( client, "sffgetpersonplus_1" );
	   return 2;
       }
    personPlus = *p;
    lastGot = GOTPLUS;
    return 0;
}

int
SffinderClient::getPart( int i, float* xyz, float* q, float* l )
{
    sffWorldPart* wp = (HEAD==i) ? &personPlus.head 
       : (LHAND==i) ? &personPlus.leftHand
       : (RHAND==i) ? &personPlus.rightHand : 0;
    if( !wp )
       return 1;
    if( !wp->valid )
       return 2;
    iotools_copyVector( 3, wp->world_ref.t, xyz );
    if( q )
       iotools_copyVector( 4, wp->world_ref.q, q );
    if( l )
       iotools_copyVector( 3, wp->world_ref.s, l );
    return 0;
}

// int
// SffinderClient::getRightHand( float* x, float* y, float* z )
// {
//     sffWorldPoint* part = &person.rightHand;
//     if( !part->valid )
//        return 1;
//     *x = part->x;
//     *y = part->y;
//     *z = part->z;
//     return 0;
// }
// 
// int
// SffinderClient::getLeftHand( float* x, float* y, float* z )
// {
//     sffWorldPoint* part = &person.leftHand;
//     if( !part->valid )
//        return 1;
//     *x = part->x;
//     *y = part->y;
//     *z = part->z;
//     return 0;
// }
// 
// int
// SffinderClient::getHead( float* x, float* y, float* z )
// {
//     sffWorldPoint* part = &person.head;
//     if( !part->valid )
//        return 1;
//     *x = part->x;
//     *y = part->y;
//     *z = part->z;
//     return 0;
// }

sffPersonPlus*
SffinderClient::getPersonPlus()
{
    sffPersonPlus* p = 0;
    if( !client )
       return 0;
    if( !( p = sffgetpersonplus_1( 0, client ) ) )
       {
	   clnt_perror( client, "sffgetpersonplus_1" );
	   return 0;
       }
    personPlus = *p;
    lastFrame = p -> frameNumber;
    lastGot = GOTPLUS;
    return &personPlus;
}

sffPersonPlus*
SffinderClient::getNextPersonPlus()
{
    sffPersonPlus* p = 0;
    if( !client )
       return 0;
    if( !( p = sffgetnextpersonplus_1( &lastFrame, client ) ) )
       {
	   clnt_perror( client, "sffgetnextpersonplus_1" );
	   return 0;
       }
    personPlus = *p;
    lastFrame = p -> frameNumber;
    lastGot = GOTPLUS;
    return &personPlus;
}

void
SffinderClient::nextUndump()
{
    int* p;
    if( !client )
       return;
    if( !( p = sffundumpnext_1( 0, client ) ) )
       {
	   clnt_perror( client, "sffnextundump_1" );
	   return;
       }
}

ostream&
SffinderClient::print( ostream& os )
{
    return os;
}

void
SffinderClient::sleep()
{
    void* p;
    if( !client )
       return;
    if( !( p = sffsleep_1( 0, client ) ) )
       {
	   clnt_perror( client, "sffsleep_1" );
       }
}

void
SffinderClient::wakeup()
{
    void* p;
    if( !client )
       return;
    if( !( p = sffwakeup_1( 0, client ) ) )
       {
	   clnt_perror( client, "sffwakeup_1" );
       }
}

int
SffinderClient::getPersonExists()
{
    int *ip;
    if( !( ip = sffgetpersonexists_1( 0, client ) ) )
       clnt_perror( client, "sffgetpersonexists_1" );
    return *ip;
}

sffPerson*
SffinderClient::getNextPerson()
{
    sffPerson* p = 0;
    if( !client )
       return 0;
    if( !( p = sffgetnextperson_1( &lastFrame, client ) ) )
       {
	   clnt_perror( client, "sffgetnextperson_1" );
	   return 0;
       }
    person = *p;
    lastFrame = p -> frameNumber;
    lastGot = GOTPERSON;
    return &person;
}
