// worldBaby.c++

#include "Parse/Parse.h++"

static int verbose = 0;

#define QmDouble
// #include "qmStatistics/qmStatistics.h++"
#include "qmNonlinearOptimize/noErfRelativeOrientation.h++"
#include "qmNonlinearOptimize/noLevenbergMarquardt.h++"

#include "PersonPlus.h++"

static
inline
void
addPoints( sffWorldPart& wp, int i, qmVectorRef& ul, qmVectorRef& vl, qmVectorRef& ur, qmVectorRef& vr )
{

    ul( i ) = wp.left.t[0];
    vl( i ) = wp.left.t[1];
    ur( i ) = wp.right.t[0];
    vr( i ) = wp.right.t[1];
}

static
int
scan( char* filename, qmVector& ul, qmVector& vl, qmVector& ur, qmVector& vr,
      int& pointCount )
{
// static
// inline
// void
// readAndCompute( char* filename,
// 		qmVectorRef& mean,
// 		qmVectorRef& direction,
// 		real* length = 0 )
// {
//     // read in points
// 
    PersonPlus personPlus;
    sffPersonPlus person;
//     qmSampleAcc xline( 3 );
//     qmGaussian xlineStats( 3 );
//     xline.setZero();
    int forceFlag = 1;
    personPlus.undump_force( filename, &forceFlag );
    int ringCount = personPlus.getRingLength();
    int featCount = 3*ringCount;
    ul.set( featCount );
    vl.set( featCount );
    ur.set( featCount );
    vr.set( featCount );

//    real* data = new real[ 3 * ringCount ];
    pointCount = 0;
    person = personPlus.get();
    int firstFrame = person.frameNumber;
    int thisFrame;
    do
       {
	   thisFrame = person.frameNumber;
	   if( person.head.valid )
	     addPoints( person.head, pointCount++, ul, vl, ur, vr );
	   if( person.leftHand.valid )
	     addPoints( person.leftHand, pointCount++, ul, vl, ur, vr );
	   if( person.rightHand.valid )
	     addPoints( person.rightHand, pointCount++, ul, vl, ur, vr );
	   do {
	       forceFlag = 1, personPlus.undump_force( filename, &forceFlag );
	       person = personPlus.get();
	   }
	   while( person.frameNumber == thisFrame );
       }
    while( person.frameNumber != firstFrame );
    cerr << pointCount << " / " << featCount << endl;
//     xline.getGaussian( xlineStats );
//     mean.copy( xlineStats.getMean() );
//     qmMatrix cov = xlineStats.getCovariance();
//     qmEigenStructure eig( cov );
//     qmMatrix evectors = eig.getEigVectors();
//     qmVector evalues = eig.getEigValues();
//     if( verbose )
//        {
// 	   cerr << "\teval= " << evalues << endl;
// 	   evectors.print( cerr << "\tevec= ", "\n\t" ) << endl;
//        }
//     int which;
//     evalues.getMax( &which );
//     direction.copy( evectors.getColumn(which) );
//     
//     if( length )
//        {
// 	   qmVector dist( pointCount );
// 	   qmVector y_ym( 3 );
// 	   for( int i = 0; i < pointCount; i++ )
// 	      dist(i) = y_ym.setSubtract
// 		 ( qmVectorRef(3,data+i*3), mean ).
// 		 getInnerProduct( direction );
// 	   *length = dist.getMax() - dist.getMin();
//        }
    return 0;
}

// void 
// setF( char* arg, void* bp )
// {
//     *(real*)bp = 1./atof( arg );
// }
// 
// void
// residualError( noErrorFunction& erf, qmVectorRef& x, ostream& os )
// {
//     static int N = 0;
//     static real rmsT = 0.;
//     qmVectorRef dy;
//     dy.set( erf( x ) );
//     // dy.print( cerr << "dy= " ) << endl;
//     real sumSquare = dy.getInnerProduct( dy );
//     int M = dy.getLength();
//     real meanSquare = sumSquare / M;
//     real rms = sqrt( meanSquare );
//     rmsT = sqrt( (N * rmsT*rmsT + M * rms*rms) / (N+M) );
//     N += M;
//     os << rms << "\t\t" << rmsT << endl;
// }
// 

int
main( int argc, char *argv[] )
{
    static real xinit[8] = { 0., 0., 0., 1., 0., 0., 0., 0. };
    char* filename = 0;

    ParseRuleFlag( 'v', verbose, 1, "Verbose mode" );
    ParseRuleArg( filename, "dump file", "filename" );
    
    if( Parse::ParseCmdLine( argc, argv ) )
       Parse::Usage( argv[0] ), exit(1);

    double tol = .001;
    double abstol = .001;
//     int vidtolMode = 0;
//     double vidtol = 1.;
    int mmtCount = 0;
//     int fixIndex = 0;
//     real fixAlpha = 0.;
    real fixAlpha = 2.;
    real beta = 1.;
//     char* residualFile = 0;
//     char* fixFilename = 0;
//     ParseRuleFlaggedArgInt( 'v', verbose, "Verbose mode" );
//     ParseRuleTaggedArgFloat( "tol", tol, "Gradient Tolerance" );
//     ParseRuleTaggedArgFloat( "abstol", abstol, "Absolute Tolerance" );
//     ParseRuleTaggedArgInt( "n", mmtCount, "Number of mmts" );
//     ParseRuleTaggedArgFloat( "init", xinit[0], "tX" );
//     ParseRuleTaggedArgFloat( "init", xinit[1], "tY" );
//     ParseRuleTaggedArgFloat( "init", xinit[2], "tZB" );
//     ParseRuleTaggedArgFloat( "init", xinit[3], "q0" );
//     ParseRuleTaggedArgFloat( "init", xinit[4], "qX" );
//     ParseRuleTaggedArgFloat( "init", xinit[5], "qY" );
//     ParseRuleTaggedArgFloat( "init", xinit[6], "qZ" );
//     ParseRuleTaggedArgFloat( "init", fixAlpha, "a" );
//     ParseRuleTaggedArgInt( "fix", fixIndex, "fixed point index" );
//     ParseRuleTaggedArgFloat( "fix", fixAlpha, "fixed point depth" );
//     ParseRuleTaggedArgFloat( "b", beta, "inverse focal length" );
//     ParseRuleTaggedArg( "f", setF, &beta, "focal length" );
//     ParseRuleTaggedArg( "res", residualFile, "residual filename" );
//     ParseRuleTaggedArg( "fixfile", fixFilename, "fixed point filename" );
// 
//     ParseRuleTag( "vidtol", vidtolMode, 1, "video tolerance mode" );
//     ParseRuleTaggedArgFloat( "vidtol", vidtol, "tol pixels" );
// 
//     char* depthOutFile = 0;
//     ParseRuleTaggedArg( "dumpDepth", depthOutFile, "depth file" );
//     char* depthInFile = 0;
//     ParseRuleTaggedArg( "undumpDepth", depthInFile, "depth input file" );
//     // int rhflag = 1;
//     // ParseRuleFlag( 'r', rhflag, 0, "rhflag" );
// 
//     if( Parse::ParseCmdLine( argc, argv ) )
//        {
// 	   Parse::Usage( argv[0] );
// 	   Parse::VerboseParse();
// 	   Parse::ParseCmdLine( argc, argv );
// 	   exit(1);
//        }

    cerr << "beta= " << beta << endl;
//     if( vidtolMode )
//        abstol = ( vidtol / 640. ) * sqrt( 2 * mmtCount );
//     cerr << "abstol= " << abstol << endl;

    noErfRelativeOrientation erf;
    qmVector uL;
    qmVector vL;
    qmVector uR;
    qmVector vR;
    if( scan( filename, uL, vL, uR, vR, mmtCount ) )
      cerr << "read error" << endl;
    

//     qmVector uL( mmtCount );
//     qmVector vL( mmtCount );
//     qmVector uR( mmtCount );
//     qmVector vR( mmtCount );
    cerr << "mmtCount= " << mmtCount << endl;
    qmVector actualState( 7 + mmtCount );
    qmVectorRef trel( 3, actualState.getArray() );
    qmUnitQuaternionRef qrel
       ( actualState.getArray( noErfRelativeOrientation::Q0 ) );
    qmVectorRef alpha
       ( mmtCount, actualState.getArray( noErfRelativeOrientation::ALPHA ) );
    qmVector state;
    state.set( 7 + mmtCount );
    state( noErfRelativeOrientation::TX ) = xinit[0];
    state( noErfRelativeOrientation::TY ) = xinit[1];
    state( noErfRelativeOrientation::TZB ) = xinit[2];
    state( noErfRelativeOrientation::Q0 ) = xinit[3];
    state( noErfRelativeOrientation::QX ) = xinit[4];
    state( noErfRelativeOrientation::QY ) = xinit[5];
    state( noErfRelativeOrientation::QZ ) = xinit[6];
    for( int i = 0; i < mmtCount; i++ )
      state( noErfRelativeOrientation::ALPHA + i ) = fixAlpha;
//     if( depthInFile )
//        {
// 	   ifstream dis( depthInFile );
// 	   for( int i = 0; i < mmtCount; i++ )
// 	      dis >> state( noErfRelativeOrientation::ALPHA + i );
// 	   dis.close();
//        }

    noLevenbergMarquardt estimator;
//     ofstream residualStream;
//     if( residualFile )
//        {
// 	   residualStream.open( residualFile );
// 	   residualStream << "RMS frame\t\tRMS total" << endl;
//        }
    
    erf.init( uL.subVector( 0, mmtCount ), 
	      vL.subVector( 0, mmtCount ),
	      uR.subVector( 0, mmtCount ), 
	      vR.subVector( 0, mmtCount ) );
    erf.setBeta( beta );
    erf.setVerbose( verbose );
// 	   if( fixFilename )
// 	      {
// 		  int indices[ 100 ];
// 		  ifstream fixFile( fixFilename );
// 		  if( fixFile.fail() )
// 		     cerr << "Whoops" << endl;
// 		  int fixCount = 0;
// 		  while( fixFile >> indices[ fixCount ] )
// 		     fixCount++;
// 		  if( fixCount <= 0 )
// 		     {
// 			 indices[0] = 0;
// 			 erf.setFixedPoints( 1, indices );
// 		     }
// 		  else
// 		     erf.setFixedPoints( fixCount, indices );
// 		  fixFile.close();
// 	      }
// 	      
    for( i = 0; i < mmtCount; i++ )
       cout << "\t" << uL(i) << "   " << vL(i) << endl;
    cerr << "starting estimator" << endl;
    estimator.init( erf );
    estimator.setGradientTolerance( tol );
    estimator.setAbsoluteTolerance( abstol );
    estimator.setState( state );
    estimator.setVerbose( verbose );
    state = estimator.estimate();
// 	  if( residualFile )
// 	    residualError( erf, state, residualStream );
    if(verbose>5) cerr << "END of estimate" << endl;
    cout << state << endl;
    cout << "END" << endl;

//     if( depthOutFile )
//       {
// 	  ofstream dos( depthOutFile );
// 	  for( int i = 0; i < mmtCount; i++ )
// 	    dos << state( noErfRelativeOrientation::ALPHA + i ) << endl;
// 	  dos.close();
//       }
//     if( residualFile )
//       residualStream.close();
    return 0;
}
