// cgTriangleStereo.c++

#include "cgTriangleStereo.h++"

cgTriangleStereo::cgTriangleStereo()
  : cgTriangle()
{
    tL[0] = 0.;
    tL[1] = 0.;
    tL[2] = 0.;
    qL[0] = 1.;
    qL[1] = 0.;
    qL[2] = 0.;
    qL[3] = 0.;
    tR[0] = 1.;
    tR[1] = 0.;
    tR[2] = 0.;
    qR[0] = 1.;
    qR[1] = 0.;
    qR[2] = 0.;
    qR[3] = 0.;
}

void
cgTriangleStereo::computeTriangle( real* TL, real* QL, real* TR, real* QR,
				   real* t, real* q )
{
    qmUnitQuaternion qRi;
    qmMatrix RRi( 3, 3 );
    qmVector tL_minus_tR( 3 );
    qRi.setInvert( QR );
    qRi.getRotationMatrix( RRi );
    qmUnitQuaternionRef(q).setMultiply( qRi, QL );
    tL_minus_tR.setSubtract( qmVectorRef( 3, TL), qmVectorRef( 3, TR ) );
    qmVectorRef( 3, t ).setMultiply( RRi, tL_minus_tR );
}

void
cgTriangleStereo::computeTriangle()
{
#if 0
    qmMatrix Rleft( 3, 3 );
    qmMatrix Rright( 3, 3 );
    qmUnitQuaternionRef( qL ).getRotationMatrix( Rleft );
    qmUnitQuaternionRef( qR ).getRotationMatrix( Rright );
    qmVectorRef tLeft( 3, tL );
    qmVectorRef tRight( 3, tR );
    qmMatrix Rrl( 3, 3 );
    // Rright.setInvertEq();
    // Rrl.setMultiply( Rright, Rleft );
    Rrl.setTMultiply( Rright, Rleft );
    qmVector Trl( 3 );
    qmVector Trlw( 3 );
    Trl.setTMultiply( Rright, Trlw.setSubtract( tLeft, tRight ) );
    Translation().copy( Trl );
    Rotation().copy( Rrl );
#endif
    computeTriangle( tL, qL, tR, qR, cgTriangle::T, cgTriangle::q );
    cgTriangle::q.getRotationMatrix( cgTriangle::R );
}

void
cgTriangleStereo::setLeftCoordinateTransform( real tx, real ty, real tz,
					      real q0, real q1,
					      real q2, real q3 )
{
    static qmMatrix R( 3, 3 );
    static qmVector t( 3 );
    t(0) = tx;
    t(1) = ty;
    t(2) = tz;
    qL[0] = q0;
    qL[1] = -q1;
    qL[2] = -q2;
    qL[3] = -q3;
    qmUnitQuaternionRef( qL ).getRotationMatrix( R );
    qmVectorRef( 3, tL ).setMultiply( R, t ).setNegateEq();
    computeTriangle();
}

void
cgTriangleStereo::setRightCoordinateTransform( real tx, real ty, real tz,
					       real q0, real q1,
					       real q2, real q3 )
{
    static qmMatrix R( 3, 3 );
    static qmVector t( 3 );
    t(0) = tx;
    t(1) = ty;
    t(2) = tz;
    qR[0] = q0;
    qR[1] = -q1;
    qR[2] = -q2;
    qR[3] = -q3;
    qmUnitQuaternionRef( qR ).getRotationMatrix( R );
    qmVectorRef( 3, tR ).setMultiply( R, t ).setNegateEq();
    computeTriangle();
}

