// cgTriangleStereo.h++

#ifndef _cgTriangleStereo_H_
#define _cgTriangleStereo_H_

#include <stream.h>
#include "qmComputeGeometry/cgTriangle.h++"

class cgTriangleStereo : public cgTriangle
{
private:
    // cgTriangle* triangle;
    // these are reference transforms (not coordinate transforms)
    real tL[3];
    real qL[4];
    real tR[3];
    real qR[4];
    void computeTriangle();
public:
    cgTriangleStereo();
    inline real point( qmVectorRef& p2L, qmVectorRef& p2R, qmVectorRef& p3,
		       real* residual = 0 );
#if 0
    inline void ellipse( qmVectorRef& mL, qmMatrixRef& CL,
			 qmVectorRef& mR, qmMatrixRef& CR,
			 qmVectorRef& m3out, qmMatrixRef& C3out );
#endif
    void setLeftCoordinateTransform( real, real, real,
				     real, real, real, real );
    inline void setLeftReferenceTransform( real, real, real,
					    real, real, real, real );
    inline void setLeftBeta( real );
    void setRightCoordinateTransform( real, real, real,
				      real, real, real, real );
    inline void setRightReferenceTransform( real, real, real,
					    real, real, real, real );
    inline void setRightBeta( real );
    inline ostream& print( ostream& );
    static void computeTriangle( real* tl, real* ql,
				 real* tr, real* qr,
				 real* t, real* q );
};

inline
real
cgTriangleStereo::point( qmVectorRef& p2L, qmVectorRef& p2R, qmVectorRef& p3,
			 real* residual )
{
    return Point( p2L(0), p2L(1), p2R(0), p2R(1), p3, residual );
}

#if 0
inline
void
cgTriangleStereo::ellipse( qmVectorRef& mL, qmMatrixRef& CL,
			 qmVectorRef& mR, qmMatrixRef& CR,
			 qmVectorRef& m3out, qmMatrixRef& C3out )
{
     ellipse( mL, CL, mR, CR, m3out, C3out );
}
#endif

inline
void
cgTriangleStereo::setLeftReferenceTransform
( real tx, real ty, real tz, real q0, real q1, real q2, real q3 )
{
    tL[0] = tx;
    tL[1] = ty;
    tL[2] = tz;
    qL[0] = q0;
    qL[1] = q1;
    qL[2] = q2;
    qL[3] = q3;
    computeTriangle();
}

inline
void
cgTriangleStereo::setLeftBeta( real b )
{
    BL() = b;
}

inline
void
cgTriangleStereo::setRightReferenceTransform
( real tx, real ty, real tz, real q0, real q1, real q2, real q3 )
{
    tR[0] = tx;
    tR[1] = ty;
    tR[2] = tz;
    qR[0] = q0;
    qR[1] = q1;
    qR[2] = q2;
    qR[3] = q3;
    computeTriangle();
}

inline
void
cgTriangleStereo::setRightBeta( real b )
{
    BR() = b;
}

inline
ostream&
cgTriangleStereo::print( ostream& os )
{
    qmVectorRef( 3, tL ).print( os << "tL= " ) << endl;
    qmVectorRef( 4, qL ).print( os << "qL= " ) << endl;
    qmVectorRef( 3, tR ).print( os << "tR= " ) << endl;
    qmVectorRef( 4, qR ).print( os << "qR= " ) << endl;
    Translation().print( os << "T= " ) << endl;
    Rotation().print( os << "R= " );
    os << "BL= " << BL() << endl;
    os << "BR= " << BR() << endl;
    return os;
}

#endif /* _cgTriangleStereo_H_ */
