// noLevenbergMarquardt.h++

#ifndef _noLevenbergMarquardt_H_
#define _noLevenbergMarquardt_H_

#include <math.h>
#include "qmNonlinearOptimize/noOptimizer.h++"
#include "qmNonlinearOptimize/noErrorFunction.h++"
#include "QuickMath/QuickMath.h++"

class noLevenbergMarquardt : public noOptimizer
{
    static real defaultU;
    static real defaultV;
    static real defaultTolerance;
    static real defaultAbsTolerance;
protected:
    noErrorFunction* erf;
    real u;
    real v;
    qmVector dy;
    qmVector Rivec;
    qmVector x;
    qmMatrix J;
    qmVector dx;
    qmVector new_x;
    qmVector new_dy;
    real new_loss;
    int reduced;
    real loss;
    qmVector JtDy;
    qmVector RiDy;
    qmMatrix RiJ;
    real magJtDy;
    real gradientTolerance;
    real absoluteTolerance;
    qmVector solveDx;
    qmMatrix solveH;
    qmMatrix solveJtJ;
    qmMatrix solveuI;
    qmVector solveJtDy;
    int verbose;
    int iterationLimit;
    int echoExitStats;
    int estimateStatus;
    int solveStatus;
protected:
    real computeLoss( qmVectorRef& dy );
    const qmVectorRef& solve( qmMatrixRef& J, qmVectorRef& dy, real u );
public:
    enum { SolveSuccess, SolveFail };
    enum { None, Success, Overflow, Singular };
    noLevenbergMarquardt();
    void init( noErrorFunction& a );
    const qmVectorRef& estimate();
    inline void setGradientTolerance( real );
    inline void setAbsoluteTolerance( real );
    inline void setRelativeTolerance( const qmVectorRef& );
    inline void setState( const qmVectorRef& );
    inline void setVerbose( int v = 1 ) { verbose = v; }
    inline void setIterationLimit( int l ) { iterationLimit = l; }
    inline void setEchoExitStats( int e = 1 ) { echoExitStats = e; }
    inline int getEstimateStatus() { return estimateStatus; }
    inline int estimateOK() { return( (Success==estimateStatus) ? 1 : 0 ); }
};

inline
void
noLevenbergMarquardt::setGradientTolerance( real tol )
{
    gradientTolerance = tol;
}

inline
void
noLevenbergMarquardt::setAbsoluteTolerance( real tol )
{
    absoluteTolerance = tol;
}

inline
void
noLevenbergMarquardt::setRelativeTolerance( const qmVectorRef& ri )
{
    Rivec.copy( ri );
}

inline
void
noLevenbergMarquardt::setState( const qmVectorRef& x_set )
{
    x = x_set;
}

#endif /* _noLevenbergMarquardt_H_ */
