// qmGaussianInvRef.h++

#ifndef _qmGaussianInvRef_H_
#define _qmGaussianInvRef_H_

#include "QuickMath/QuickMath.h++"

class qmGaussianInvRef
{
    static ostream& alert( ostream&, char* msg );
    inline real _getMahalanobisSquare( const qmVectorRef& ) const;
    inline real _getLogLikelihood( const qmVectorRef& ) const;
protected:
    inline int _testSingular() const;
    inline void testSetSingular();

    unsigned int state;
    qmVectorRef mean;
    qmMatrixRef invCovariance;

public:

    static real MinDet;  // distribution considered singular if det < MinDet
    static int getStorageSize( int n ) { return( sizeof(real)*n*(n+1) ); }
    enum { NONSINGULAR=0, SINGULAR=1, NOT_INITED=2, NAN=3 }; // NAN not impl

    qmGaussianInvRef()
      : state( NOT_INITED ), mean( ), invCovariance( ) {}
    qmGaussianInvRef( const int dim, real* a )
      : state( NOT_INITED ), mean( dim, a ), invCovariance( dim, dim, a+dim )
    {}
    qmGaussianInvRef( const qmGaussianInvRef& g )
      : state( g.state ), mean( g.mean ), invCovariance( g.invCovariance ) {}
    qmGaussianInvRef( const qmVectorRef& v, const qmMatrixRef& m )
      : state( NOT_INITED ), mean( v ), invCovariance( m ){ testSetSingular();}

    void set( const int dim, real* a );
    void set( const qmGaussianInvRef& );
    void copy( const qmGaussianInvRef& );
			  
    inline int getDim() const { return mean.getLength(); }
    const qmVectorRef& getMean() const { return mean; } 
    void setMean( const qmVectorRef & v ) { mean.copy( v ); }
    const qmMatrixRef& getInvCovariance() const { return invCovariance; }
    void setInvCovariance( const qmMatrixRef& m );
    inline void set( const qmVectorRef&, const qmMatrixRef& );
    int getState() const { return state; }
    int isSingular() const { return( SINGULAR == state ); }

    real getMahalanobisSquare( const qmVectorRef& ) const;
    real getMahalanobis( const qmVectorRef& ) const;
    real getLikelihood( const qmVectorRef& ) const;
    real getLogLikelihood( const qmVectorRef& ) const;

    ostream& write( ostream& );
    istream& read( istream& );
    ostream& printPretty( ostream&, char* = "\n" );
    // istream& scan( istream& );
};

inline
void
qmGaussianInvRef::set( const qmVectorRef& v, const qmMatrixRef& m )
{
    setMean( v );
    setInvCovariance( m );
}

inline
int
qmGaussianInvRef::_testSingular() const
{
    if( invCovariance.getDeterminant() < MinDet )
       return 1;
    return 0;
}

inline
void
qmGaussianInvRef::testSetSingular()
{
    if( _testSingular() )
       state = SINGULAR;
    else
       state = NONSINGULAR;
}

#endif /* _qmGaussianInvRef_H_ */
