// vlImage.h++

#ifndef _vlImage_H_
#define _vlImage_H_

#include "vlImage/vliImageRef.h++"
#include "vlImage/vliPixel.h++"
#include "vlImage/vliRGBImage.h++"
#include "vlImage/vliRGBImageRef.h++"
#include "vlImage/vliRGBPixel.h++"
#include "vlImage/vliTypes.h++"
#include "vlImage/vliYUVImage.h++"
#include "vlImage/vliYUVImageRef.h++"
#include "vlImage/vliYUVPixel.h++"

#include <vl/vl.h>

class vlImage
{
    static VLServer server;	// Set by Init
    static VLNode drain;	// Set by Init
    static VLNode source;	// Set by Init
    static VLPath path;		// Set by Init
    static VLBuffer buffer;	// Set by Init
    static VLInfoPtr info;
    static VLControlValue val;
    static int transferBegun;	// Set by Init and Close
    static int captureWidth;	// Set by Zoom
    static int captureHeight;	// Set by Zoom
    static int captureSize;	// Set by Zoom (height * width)
    static int captureType;
    static int captureBytesPerPixel;
    static int verbose;
 public:
    enum { QUERY = -1, RGB = 0, YUV = 1 };
    enum { ANALOG = 0, DIGITAL = 1 };
    enum { SVIDEO_1 = 0, SVIDEO_2 = 1, SVIDEO_3 = 2,
	       COMPOSITE_1 = 3, COMPOSITE_2 = 4, COMPOSITE_3 = 5,
	       YRB_1 = 6, YRB_2 = 7 };

    static void init( int input = SVIDEO_1,
		      int zoom = 1,
		      int encoding = RGB,
		      int aord = ANALOG,
		      int nbuffers = 4 );
    static void close();
    static vliImageRef blindGrab();
    static void grab( vliRGBImageRef& );
    static void grab( vliYUVImageRef& );
    static void free();
    static void setEncoding( int );
    static int getEncoding( );
    static void setZoom( int );
    static int getZoom( );
    static void setInput( int );
    static int getInput( );
    static void setVerbose( int );
    static inline void yuvToRgb( const vliYUVPixel&, vliRGBPixel& );
    static void yuvToRgb( const vliYUVImageRef&, vliRGBImageRef& );
    static void yuvToRgb2( const vliYUVImageRef&, vliRGBImageRef& );
    static inline void rgbToYuv( const vliRGBPixel&, vliYUVPixel& );
    static void rgbToYuv( const vliRGBImageRef&, vliYUVImageRef& );
};

inline
void
vlImage::yuvToRgb( const vliYUVPixel& yuv, vliRGBPixel& rgb )
{
    vliPixel::yuvToRgb( yuv.y, yuv.u, yuv.v, rgb.r(), rgb.g(), rgb.b() );
}

inline
void
vlImage::rgbToYuv( const vliRGBPixel& rgb, vliYUVPixel& yuv )
{
    vliPixel::rgbToYuv( rgb.getR(), rgb.getG(), rgb.getB(),
			yuv.y, yuv.u, yuv.v );
}

#endif /* _vlImage_H_ */
