// vliYUVImageRef.c++

#include "vlImage/vliYUVImageRef.h++"
#include "vlImage/vliPixel.h++"
#include <math.h>

istream&
vliYUVImageRef::readRawRGB( istream& is )
{
    byte rgb[6];
    byte* uyvy_p = data;
    for( int i = 0; i < pixelCount; i += 2 )
       {
	   // process 2 pixels at a time
	   is.read( rgb, 6 );
	   if( is.eof() )
	      return is;
	   vliPixel::rgb2ToUyvy( rgb[0], rgb[1], rgb[2],
				 rgb[3], rgb[4], rgb[5],
				 uyvy_p[0], uyvy_p[1], uyvy_p[2], uyvy_p[3] );
	   uyvy_p += 4;
       }
    return is;
}

ostream&
vliYUVImageRef::writeRawRGB( ostream& os )
{
    byte rgb[6];
    byte* uyvy_p = data;
    for( int i = 0; i < pixelCount; i += 2 )
       {
	   // process 2 pixels at a time
	   vliPixel::yuvToRgb( uyvy_p[1], uyvy_p[0], uyvy_p[2],
			       rgb[0], rgb[1], rgb[2] );
	   vliPixel::yuvToRgb( uyvy_p[3], uyvy_p[0], uyvy_p[2],
			       rgb[3], rgb[4], rgb[5] );
	   uyvy_p += 4;
	   os.write( rgb, 6 );
	   // os.write( uyvy_p, 6 );
       }
    os.flush();
    return os;
}

void
vliYUVImageRef::black()
{
    byte* bp = data;
    int count = pixelCount;
    while( count-- )
       {
	   *bp++ = 128;
	   *bp++ = 16;
       }
}

#if 0

vliYUVPixel
vliYUVImageRef::getPixel( int i )
{
#ifdef vliYUVImageRefDEBUG
    if( i < 0 || i >= pixelCount )
       cerr << "vliYUVImageRef::getPixel: DEBUG: boundary error" << endl;
#endif
    vliYUVPixel pixel;
    pixel.y = getY(i);
    pixel.u = getU(i);
    pixel.v = getV(i);
    return pixel;
}

#endif

void
vliYUVImageRef::drawBlob( double um, double vm,
			  double c11, double c22, double c12,
			  vliYUVPixel& color, double d )
{
    double determ = c11*c22 - c12*c12;
    int u_plus;
    int u_minus;
    int v_plus;
    int v_minus;
    double discrim;
    double s;
    double t;
    for( double du = 0.; ( discrim = c22*d - du*du*determ ) >= 0.; du += 1. )
       {
	   s = -c12*du/c22;
	   t = sqrt( discrim ) / c22;
	   u_plus = um + int( du );
	   u_minus = um - int( du );
	   v_minus = vm + int( s - t );
	   v_plus = vm + int( s + t );
	   for( int v = v_minus; v <= v_plus; v++ )
	      setPixelSafe( u_plus, v, color );
	   if( !du )
	      continue;
	   v_minus = vm + int( -s - t );
	   v_plus = vm + int( -s + t );
	   for( v = v_minus; v <= v_plus; v++ )
	      setPixelSafe( u_minus, v, color );
       }
}

void
vliYUVImageRef::drawHLine( float u0, float u1, float v, vliYUVPixel& color )
{
    int umin = vliPixel::min( u0, u1 );
    int umax = vliPixel::max( u0, u1 );
    for( int u = umin; u <= umax; u++ )
       setPixelSafe( u, v, color );
}

void
vliYUVImageRef::drawVLine( float v0, float v1, float u, vliYUVPixel& color )
{
    int vmin = vliPixel::min( v0, v1 );
    int vmax = vliPixel::max( v0, v1 );
    for( int v = vmin; v <= vmax; v++ )
       setPixelSafe( u, v, color );
}

void 
vliYUVImageRef::drawLine( float u0, float u1,  
			  float v0, float v1, 
			  const vliYUVPixel& color)
{
  float length = fabsf(u1-u0);
  if (fabsf(v1-v0) > length )
    length = fabsf(v1-v0);
  float u = u0 + 0.5;
  float v = v0 + 0.5;
  float du = (u1-u0)/length;
  float dv = (v1-v0)/length;
  for(int i=1; i<length; i++)
    {
      setPixelSafe( u, v, color );
      u += du;
      v += dv;
    }
}

void
vliYUVImageRef::overlay( vliYUVImageRef& olay )
{
    vliYUVPixel yuv;
    for( int i = 0; i < pixelCount; i++ )
       {
	   yuv = olay.getPixel( i );
	   if( !yuv.isBlack() )
	      setPixel( i, yuv );
       }
}

void
vliYUVImageRef::map( void (*fn)( vliYUVPixel& ) )
{
    vliYUVPixel yuv;
    for( int i = 0; i < pixelCount; i++ )
       {
	   yuv = getPixel( i );
	   fn( yuv );
	   setPixel( i, yuv );
       }
}

float vliYUVImageRef::toas = .9;
float vliYUVImageRef::toao = 30.;

#define LOOK_GOOD

void
vliYUVImageRef::_toabgr( byte* p ) const
{
    register float BLU;
    register float GRN;
    register float RED;
    byte* uyvy = data;
    float A1;
    float A2;
    float B;
    float C;
    float D;
    float E;

    int height = pixelCount / width;
    byte* rowsrc = data;
    byte* rowdst = p;
    int nbytes = width*4;
    int nbytes2 = width*8;
    int nbytes3 = width*12;
    int nbytes4 = width*16;

    for( int j = 0; j < height; j += 4 )
       {
	   uyvy = rowsrc;
	   p = rowdst;
	   for( int i = 0; i < width; i += 4 )
	      {
		  A1 = 1.164*(uyvy[1]-16.0);
		  A2 = 1.164*(uyvy[3]-16.0);
		  B = 2.017*(uyvy[0]-128.0);
		  C = -0.392*(uyvy[0]-128.0);
		  D = -0.813*(uyvy[2]-128.0);
		  E = 1.596*(uyvy[2]-128.0);
		  BLU = toas*( A1 + B + toao );
		  GRN = toas*( A1 + C + D + toao );
		  RED = toas*( A1 + E + toao );
#ifdef LOOK_GOOD
		  BLU = vliPixel::limit( BLU, 0, 255 );
		  GRN = vliPixel::limit( GRN, 0, 255 );
		  RED = vliPixel::limit( RED, 0, 255 );
#endif
		  p++;
		  *p++ = BLU;
		  *p++ = GRN;
		  *p++ = RED;
		  p++;
		  *p++ = BLU;
		  *p++ = GRN;
		  *p++ = RED;
		  p++;
		  *p++ = BLU;
		  *p++ = GRN;
		  *p++ = RED;
		  p++;
		  *p++ = BLU;
		  *p++ = GRN;
		  *p++ = RED;
		  uyvy += 8;
	      }
	   bcopy( rowdst, rowdst+nbytes, nbytes );
	   bcopy( rowdst, rowdst+nbytes2, nbytes );
	   bcopy( rowdst, rowdst+nbytes3, nbytes );
	   rowdst += nbytes4;
	   rowsrc += nbytes2;
       }
}
